@@ -68,6 +68,18 @@
 #ifndef __GLOBALS__
 #define __GLOBALS__
 
+/* i18n */
+#include "gettext.h"
+#ifdef ENABLE_NLS
+# ifdef gettext_noop
+#  define N_(String) gettext_noop (String)
+# else
+#  define N_(String) (String)
+# endif
+#else
+# define N_(String) (String)
+#endif
+
 
 /*------------------------------------------------------------------*/
 /*!
