@@ -45,10 +45,6 @@
 #include "../include/globals.h"
 #include "../include/i_vars.h"
 
-#ifdef HAVE_LIBDMALLOC
-#include <dmalloc.h>
-#endif
-
 /* GtkWidget *w_main; */
 
 
