@@ -34,18 +34,11 @@
 #include "../include/prototype.h"  /* function prototypes */
 #include "../include/globals.h"
 
-#ifdef HAVE_LIBDMALLOC
-#include <dmalloc.h>
-#endif
-
 /* command line arguments */
 int verbose_mode=FALSE; //!< Reflects the value of the command line flag
 int quiet_mode=FALSE;   //!< Reflects the value of the command line flag
 
 /*!
  * these are required by libgeda
- * I have made most of these NULL because they aren't needed
- * for gattrib -- no drawing is done.
  */
 void (*variable_set_func)() = i_vars_set;
-
