@@ -1,21 +1,21 @@
 /* gEDA - GPL Electronic Design Automation
  * gschem - gEDA Schematic Capture
  * Copyright (C) 1998-2010 Ales Hvezda
+ * Copyright (C) 1998-2019 gEDA Contributors (see ChangeLog for details)
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- * MA 02111-1301 USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -37,10 +37,10 @@ struct _GschemDialogClass {
   GtkDialogClass parent_class;
 
   void (*geometry_save)    (GschemDialog *dialog,
-                            GKeyFile *key_file,
+                            EdaConfig *cfg,
                             gchar *group_name);
   void (*geometry_restore) (GschemDialog *dialog,
-                            GKeyFile *key_file,
+                            EdaConfig *cfg,
                             gchar *group_name);
 };
 
@@ -48,14 +48,14 @@ struct _GschemDialog {
   GtkDialog parent_instance;
 
   gchar *settings_name;
-  GSCHEM_TOPLEVEL *w_current;
+  GschemToplevel *w_current;
 };
 
 
 GType gschem_dialog_get_type (void);
 
 GtkWidget* gschem_dialog_new_with_buttons (const gchar *title, GtkWindow *parent, GtkDialogFlags flags,
-                                           const gchar *settings_name, GSCHEM_TOPLEVEL *w_current,
+                                           const gchar *settings_name, GschemToplevel *w_current,
                                            const gchar *first_button_text, ...);
 
 
