@@ -2,57 +2,37 @@
 extern int default_text_size;
 extern int default_text_caps;
 extern gchar *default_print_command;
-extern int default_override_net_color;
-extern int default_override_bus_color;
-extern int default_override_pin_color;
-extern int default_net_endpoint_mode;
-extern int default_net_midpoint_mode;
 extern int default_net_direction_mode;
 extern int default_net_selection_mode;
-extern int default_line_style;
-extern int default_pin_style;
-extern int default_net_style;
-extern int default_bus_style;
-extern int default_background_color;
 extern int default_zoom_with_pan;
 extern int default_actionfeedback_mode;
-extern int default_text_feedback;
-extern int default_text_display_zoomfactor;
-extern int default_object_clipping;
 extern int default_do_logging;
 extern int default_logging_dest;
 extern int default_embed_complex;
 extern int default_include_complex;
 extern int default_snap_size;
 
-extern int default_paper_width;
-extern int default_paper_height;
 extern int default_init_right;
 extern int default_init_bottom;
 extern int default_scrollbars_flag;
-extern int default_print_output_type;
+extern gchar *default_print_paper;
 extern int default_print_orientation;
 extern int default_print_color;
-extern int default_print_color_background;
-extern int default_print_output_capstyle;
 extern int default_image_color;
 extern int default_image_width;
 extern int default_image_height;
-extern int default_width;
-extern int default_height;
 extern int default_log_window;
 extern int default_log_window_type;
 extern int default_third_button;
+extern int default_third_button_cancel;
 extern int default_middle_button;
 extern int default_scroll_wheel;
 extern int default_net_consolidate;
 extern int default_file_preview;
 extern int default_enforce_hierarchy;
-extern int default_text_origin_marker;
 extern int default_fast_mousepan;
 extern int default_raise_dialog_boxes;
 extern int default_continue_component_place;
-extern GList *default_component_select_attrlist;
 extern int default_undo_levels;
 extern int default_undo_control;
 extern int default_undo_type;
@@ -60,7 +40,6 @@ extern int default_undo_panzoom;
 extern int default_draw_grips;
 extern int default_netconn_rubberband;
 extern int default_magnetic_net_mode;
-extern int default_sort_component_library;
 extern int default_warp_cursor;
 extern int default_toolbars;
 extern int default_handleboxes;
@@ -70,10 +49,12 @@ extern int default_bus_ripper_size;
 extern int default_bus_ripper_type;
 extern int default_bus_ripper_rotation;
 extern int default_force_boundingbox;
+extern int default_grid_mode;
 extern int default_dots_grid_dot_size;
 extern int default_dots_grid_mode;
 extern int default_dots_grid_fixed_threshold;
 extern int default_mesh_grid_display_threshold;
+extern int default_show_origin;
 extern int default_add_attribute_offset;
 extern int default_auto_save_interval;
 extern int default_mousepan_gain;
