@@ -1,7 +1,7 @@
 /* gEDA - GPL Electronic Design Automation
  * gschem - gEDA Schematic Capture
  * Copyright (C) 1998-2010 Ales Hvezda
- * Copyright (C) 1998-2011 gEDA Contributors (see ChangeLog for details)
+ * Copyright (C) 1998-2019 gEDA Contributors (see ChangeLog for details)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,16 +38,99 @@
 
 #include "gschem.h"
 
-#ifdef HAVE_LIBDMALLOC
-#include <dmalloc.h>
-#endif
+/*! \brief Start the process of selection
+ *  \par Function Description
+ *  Chooses the way of how to start the selection process. If no
+ *  grip was found at the given coordinates the function sets
+ *  \a w_current->inside_action in order to force other functions
+ *  (\a o_select_motion() or \a o_select_end()) to decide that.
+ *  Otherwise, it switches on the GRIPS mode for working with the
+ *  grip found.
+ *
+ *  The function is intended to be called by pressing the left
+ *  mouse button.
+ *
+ *  \param [in] w_current The GschemToplevel structure.
+ *  \param [in] wx        The world X coordinate.
+ *  \param [in] wy        The world Y coordinate.
+ */
+void o_select_start (GschemToplevel *w_current, int wx, int wy)
+{
+  /* look for grips or fall through if not enabled */
+  o_grips_start(w_current, wx, wy);
+
+  if (w_current->event_state != GRIPS) {
+    /* now go into normal SELECT */
+    i_action_start (w_current);
+    w_current->first_wx = w_current->second_wx = wx;
+    w_current->first_wy = w_current->second_wy = wy;
+  }
+}
+
+/*! \brief End the process of selection
+ *  \par Function Description
+ *  Finishes the process of selection if the \a o_select_start()
+ *  or \a o_select_motion() functions haven't defined other
+ *  functions to finish it.  In this case the function tries to
+ *  find an object under the mouse pointer and select it.
+ *
+ *  The function is intended to be called by releasing the left
+ *  mouse button.
+ *
+ *  \param [in] w_current The GschemToplevel structure.
+ *  \param [in] wx        The world X coordinate.
+ *  \param [in] wy        The world Y coordinate.
+ */
+void o_select_end (GschemToplevel *w_current, int wx, int wy)
+{
+  g_assert (w_current->inside_action != 0);
+
+  /* look for objects to select */
+  o_find_object(w_current, wx, wy, TRUE);
+  i_action_stop (w_current);
+}
+
+
+/*! \brief Determine whether objects have to be selected or moved
+ *  \par Function Description
+ *  Checks if the shift or control keys are pressed, (that means
+ *  the user definitely wants to drag out a selection box), or
+ *  there are no selected objects under the cursor. In that case
+ *  the function starts drawing the selection box. Otherwise, it
+ *  looks for the objects that have been or could be selected and
+ *  starts moving them.
+ *
+ *  The function is intended to be called by motion of the mouse
+ *  while the left mouse button is pressed.
+ *
+ *  \param [in] w_current The GschemToplevel structure.
+ *  \param [in] wx        The world X coordinate.
+ *  \param [in] wy        The world Y coordinate.
+ */
+void o_select_motion (GschemToplevel *w_current, int wx, int wy)
+{
+  g_assert (w_current->inside_action != 0);
+
+  /* Check if a mod key is pressed or there is no selected object
+   * under the cursor */
+  if (w_current->SHIFTKEY || w_current->CONTROLKEY
+          || (!o_find_selected_object(w_current, w_current->first_wx, w_current->first_wy)
+              && (!o_find_object(w_current, w_current->first_wx, w_current->first_wy, TRUE)
+                  || !o_select_selected(w_current)))) {
+    /* Start drawing a selection box to select objects */
+    o_select_box_start(w_current, wx, wy);
+  } else {
+    /* Start moving the selected object(s) */
+    o_move_start(w_current, w_current->first_wx, w_current->first_wy);
+  }
+}
 
 /*! \todo Finish function documentation!!!
  *  \brief
  *  \par Function Description
  *
  */
-void o_select_run_hooks(GSCHEM_TOPLEVEL *w_current, OBJECT *o_current, int flag)
+void o_select_run_hooks(GschemToplevel *w_current, OBJECT *o_current, int flag)
 {
   switch (flag) {
   /* If flag == 0, then we are deselecting something. */
@@ -68,13 +151,13 @@ void o_select_run_hooks(GSCHEM_TOPLEVEL *w_current, OBJECT *o_current, int flag)
  *  \par Function Description
  *
  *  \note
- *  type can be either SINGLE meaning selection is a single mouse click 
+ *  type can be either SINGLE meaning selection is a single mouse click
  *      or it can be MULTIPLE meaning selection is a selection box
  */
-void o_select_object(GSCHEM_TOPLEVEL *w_current, OBJECT *o_current,
+void o_select_object(GschemToplevel *w_current, OBJECT *o_current,
 		     int type, int count)
 {
-  TOPLEVEL *toplevel = w_current->toplevel;
+  TOPLEVEL *toplevel = gschem_toplevel_get_toplevel (w_current);
   int SHIFTKEY;
   int CONTROLKEY;
   int removing_obj = 0;
@@ -82,7 +165,7 @@ void o_select_object(GSCHEM_TOPLEVEL *w_current, OBJECT *o_current,
   SHIFTKEY = w_current->SHIFTKEY;
   CONTROLKEY = w_current->CONTROLKEY;
 
-#if DEBUG 
+#if DEBUG
   printf("OBJECT id: %d\n", o_current->sid);
 #endif
 
@@ -147,7 +230,7 @@ void o_select_object(GSCHEM_TOPLEVEL *w_current, OBJECT *o_current,
             o_select_run_hooks( w_current, o_current, 1 );
             o_selection_add (toplevel,
                              toplevel->page_current->selection_list, o_current);
-          }	
+          }
 
           /* condition: doing single object add */
           /* condition: control key not pressed */
@@ -169,7 +252,7 @@ void o_select_object(GSCHEM_TOPLEVEL *w_current, OBJECT *o_current,
           }
 
           break;
-      } 
+      }
       break; /* end object selected switch */
   }
 
@@ -204,22 +287,29 @@ void o_select_object(GSCHEM_TOPLEVEL *w_current, OBJECT *o_current,
  *  \par Function Description
  *
  */
-int o_select_box_start(GSCHEM_TOPLEVEL *w_current, int w_x, int w_y)
+void o_select_box_start(GschemToplevel *w_current, int w_x, int w_y)
 {
-  int diff_x, diff_y;
+  g_return_if_fail (w_current != NULL);
+
+  GschemPageView *page_view = gschem_toplevel_get_current_page_view (w_current);
+  g_return_if_fail (page_view != NULL);
+
+  int diff_x, diff_y, dist;
 
   diff_x = abs(w_current->first_wx - w_x);
   diff_y = abs(w_current->first_wy - w_y);
 
   /* if we are still close to the button press location,
      then don't enter the selection box mode */
-  if (SCREENabs (w_current, max(diff_x, diff_y)) < 10) {
-    return FALSE;
-  }
+  dist = gschem_page_view_SCREENabs (page_view, max(diff_x, diff_y));
 
-  w_current->second_wx = w_x;
-  w_current->second_wy = w_y;
-  return TRUE;
+  if (dist >= 10) {
+    w_current->second_wx = w_x;
+    w_current->second_wy = w_y;
+
+    i_set_state (w_current, SBOX);
+    i_action_start (w_current);
+  }
 }
 
 /*! \todo Finish function documentation!!!
@@ -227,12 +317,17 @@ int o_select_box_start(GSCHEM_TOPLEVEL *w_current, int w_x, int w_y)
  *  \par Function Description
  *
  */
-void o_select_box_end(GSCHEM_TOPLEVEL *w_current, int w_x, int w_y)
+void o_select_box_end(GschemToplevel *w_current, int w_x, int w_y)
 {
+  g_assert (w_current->inside_action != 0);
+
   o_select_box_invalidate_rubber (w_current);
   w_current->rubber_visible = 0;
 
   o_select_box_search(w_current);
+
+  i_set_state(w_current, SELECT);
+  i_action_stop (w_current);
 }
 
 /*! \todo Finish function documentation!!!
@@ -240,12 +335,14 @@ void o_select_box_end(GSCHEM_TOPLEVEL *w_current, int w_x, int w_y)
  *  \par Function Description
  *
  */
-void o_select_box_motion (GSCHEM_TOPLEVEL *w_current, int w_x, int w_y)
+void o_select_box_motion (GschemToplevel *w_current, int w_x, int w_y)
 {
+  g_assert (w_current->inside_action != 0);
+
   if (w_current->rubber_visible)
     o_select_box_invalidate_rubber (w_current);
-    
-  w_current->second_wx = w_x; 
+
+  w_current->second_wx = w_x;
   w_current->second_wy = w_y;
 
   o_select_box_invalidate_rubber (w_current);
@@ -256,17 +353,18 @@ void o_select_box_motion (GSCHEM_TOPLEVEL *w_current, int w_x, int w_y)
  *  \brief
  *  \par Function Description
  */
-void o_select_box_invalidate_rubber (GSCHEM_TOPLEVEL *w_current)
+void o_select_box_invalidate_rubber (GschemToplevel *w_current)
 {
-  int x1, y1, x2, y2;
+  g_return_if_fail (w_current != NULL);
 
-  WORLDtoSCREEN (w_current, w_current->first_wx, w_current->first_wy, &x1, &y1);
-  WORLDtoSCREEN (w_current, w_current->second_wx, w_current->second_wy, &x2, &y2);
+  GschemPageView *page_view = gschem_toplevel_get_current_page_view (w_current);
+  g_return_if_fail (page_view != NULL);
 
-  o_invalidate_rect (w_current, x1, y1, x2, y1);
-  o_invalidate_rect (w_current, x1, y1, x1, y2);
-  o_invalidate_rect (w_current, x2, y1, x2, y2);
-  o_invalidate_rect (w_current, x1, y2, x2, y2);
+  gschem_page_view_invalidate_world_rect (page_view,
+                                          w_current->first_wx,
+                                          w_current->first_wy,
+                                          w_current->second_wx,
+                                          w_current->second_wy);
 }
 
 /*! \todo Finish function documentation!!!
@@ -274,15 +372,9 @@ void o_select_box_invalidate_rubber (GSCHEM_TOPLEVEL *w_current)
  *  \par Function Description
  *
  */
-void o_select_box_draw_rubber (GSCHEM_TOPLEVEL *w_current)
+void o_select_box_draw_rubber (GschemToplevel *w_current, EdaRenderer *renderer)
 {
-  gschem_cairo_box (w_current, 0,
-                    w_current->first_wx, w_current->first_wy,
-                    w_current->second_wx, w_current->second_wy);
-
-  gschem_cairo_set_source_color (w_current,
-                                 x_color_lookup_dark (SELECT_COLOR));
-  gschem_cairo_stroke (w_current, TYPE_SOLID, END_NONE, 0, -1, -1);
+  o_box_draw_rubber (w_current, renderer);
 }
 
 /*! \todo Finish function documentation!!!
@@ -290,16 +382,16 @@ void o_select_box_draw_rubber (GSCHEM_TOPLEVEL *w_current)
  *  \par Function Description
  *
  */
-void o_select_box_search(GSCHEM_TOPLEVEL *w_current)
+void o_select_box_search(GschemToplevel *w_current)
 {
-  TOPLEVEL *toplevel = w_current->toplevel;
+  TOPLEVEL *toplevel = gschem_toplevel_get_toplevel (w_current);
   OBJECT *o_current=NULL;
   int count = 0; /* object count */
   int SHIFTKEY = w_current->SHIFTKEY;
   int CONTROLKEY = w_current->CONTROLKEY;
   int left, right, top, bottom;
   const GList *iter;
-	
+
   left = min(w_current->first_wx, w_current->second_wx);
   right = max(w_current->first_wx, w_current->second_wx);
   top = min(w_current->first_wy, w_current->second_wy);
@@ -309,12 +401,15 @@ void o_select_box_search(GSCHEM_TOPLEVEL *w_current)
   while (iter != NULL) {
     o_current = iter->data;
     /* only select visible objects */
-    if (o_is_visible (toplevel, o_current) || toplevel->show_hidden_text) {
+    if (o_is_visible (o_current) || toplevel->show_hidden_text) {
+      int cleft, ctop, cright, cbottom;
 
-      if ( o_current->w_left   >= left &&
-           o_current->w_right  <= right  &&
-           o_current->w_top    >= top  &&
-           o_current->w_bottom <= bottom ) {
+      if ( world_get_single_object_bounds(toplevel, o_current,
+                                          &cleft, &ctop, &cright, &cbottom) &&
+           cleft   >= left &&
+           cright  <= right  &&
+           ctop    >= top  &&
+           cbottom <= bottom ) {
 
         o_select_object(w_current, o_current, MULTIPLE, count);
         count++;
@@ -337,12 +432,12 @@ void o_select_box_search(GSCHEM_TOPLEVEL *w_current)
  *   and the net_selection_state of the current net this function will either
  *   select the single net, all directly connected nets or all nets connected
  *   with netname labels.
- *  \param [in] w_current  GSCHEM_TOPLEVEL struct.
+ *  \param [in] w_current  GschemToplevel struct.
  *  \param [in] o_net      Pointer to a single net object
  */
-void o_select_connected_nets(GSCHEM_TOPLEVEL *w_current, OBJECT* o_net)
+void o_select_connected_nets(GschemToplevel *w_current, OBJECT* o_net)
 {
-  TOPLEVEL *toplevel = w_current->toplevel;
+  TOPLEVEL *toplevel = gschem_toplevel_get_toplevel (w_current);
   const GList *o_iter;
   GList *iter1;
   OBJECT *o_current;
@@ -355,6 +450,15 @@ void o_select_connected_nets(GSCHEM_TOPLEVEL *w_current, OBJECT* o_net)
 
   g_assert(o_net->type == OBJ_NET);
 
+  /* If either SHIFT or CTRL are pressed, behave exactly the same as a
+   * single object selection.  This makes it possible to <mouse-1> on
+   * a net segment to select it and then Shift+<mouse-1> on it to
+   * deselect it. */
+  if (w_current->SHIFTKEY || w_current->CONTROLKEY) {
+    o_select_object (w_current, o_net, SINGLE, 0);
+    return;
+  }
+
   if (!o_net->selected) {
     w_current->net_selection_state = 1;
   }
@@ -362,14 +466,14 @@ void o_select_connected_nets(GSCHEM_TOPLEVEL *w_current, OBJECT* o_net)
   /* the current net is the startpoint for the stack */
   netstack = g_list_prepend(netstack, o_net);
 
-  count = 0; 
+  count = 0;
   while (1) {
     netnameiter = g_list_last(netnamestack);
     for (iter1 = g_list_last(netstack);
-	 iter1 != NULL; 
+	 iter1 != NULL;
 	 iter1 = g_list_previous(iter1), count++) {
       o_current = iter1->data;
-      if (o_current->type == OBJ_NET && 
+      if (o_current->type == OBJ_NET &&
 	  (!o_current->selected || count == 0)) {
 	o_select_object (w_current, o_current, SINGLE, count);
 	if (w_current->net_selection_state > 1) {
@@ -426,10 +530,10 @@ void o_select_connected_nets(GSCHEM_TOPLEVEL *w_current, OBJECT* o_net)
 }
 
 /* This is a wrapper for o_selection_return_first_object */
-/* This function always looks at the current page selection list */ 
-OBJECT *o_select_return_first_object(GSCHEM_TOPLEVEL *w_current)
+/* This function always looks at the current page selection list */
+OBJECT *o_select_return_first_object(GschemToplevel *w_current)
 {
-  TOPLEVEL *toplevel = w_current->toplevel;
+  TOPLEVEL *toplevel = gschem_toplevel_get_toplevel (w_current);
   if (! (w_current && toplevel->page_current && geda_list_get_glist( toplevel->page_current->selection_list )))
     return NULL;
   else
@@ -443,9 +547,9 @@ OBJECT *o_select_return_first_object(GSCHEM_TOPLEVEL *w_current)
  * \return TRUE if the selection list is not empty, otherwise false.
  * also make sure item is valid
  */
-int o_select_selected(GSCHEM_TOPLEVEL *w_current)
+int o_select_selected(GschemToplevel *w_current)
 {
-  TOPLEVEL *toplevel = w_current->toplevel;
+  TOPLEVEL *toplevel = gschem_toplevel_get_toplevel (w_current);
   if ( geda_list_get_glist( toplevel->page_current->selection_list )) {
     return(TRUE);
   }
@@ -458,9 +562,9 @@ int o_select_selected(GSCHEM_TOPLEVEL *w_current)
  *  \par Function Description
  *
  */
-void o_select_unselect_all(GSCHEM_TOPLEVEL *w_current)
+void o_select_unselect_all(GschemToplevel *w_current)
 {
-  TOPLEVEL *toplevel = w_current->toplevel;
+  TOPLEVEL *toplevel = gschem_toplevel_get_toplevel (w_current);
   SELECTION *selection = toplevel->page_current->selection_list;
   GList *removed = NULL;
   GList *iter;
@@ -482,12 +586,12 @@ void o_select_unselect_all(GSCHEM_TOPLEVEL *w_current)
  * unlocked on the current page, and any attached attributes whether
  * visible or invisible..
  *
- * \param w_current  The current #GSCHEM_TOPLEVEL structure.
+ * \param w_current  The current #GschemToplevel structure.
  */
 void
-o_select_visible_unlocked (GSCHEM_TOPLEVEL *w_current)
+o_select_visible_unlocked (GschemToplevel *w_current)
 {
-  TOPLEVEL *toplevel = w_current->toplevel;
+  TOPLEVEL *toplevel = gschem_toplevel_get_toplevel (w_current);
   SELECTION *selection = toplevel->page_current->selection_list;
   const GList *iter;
   GList *added;
@@ -499,7 +603,7 @@ o_select_visible_unlocked (GSCHEM_TOPLEVEL *w_current)
     OBJECT *obj = (OBJECT *) iter->data;
 
     /* Skip invisible objects. */
-    if (!o_is_visible (toplevel, obj) && !toplevel->show_hidden_text)
+    if (!o_is_visible (obj) && !toplevel->show_hidden_text)
       continue;
 
     /* Skip locked objects. */
@@ -530,9 +634,9 @@ o_select_visible_unlocked (GSCHEM_TOPLEVEL *w_current)
  *
  */
 void
-o_select_move_to_place_list(GSCHEM_TOPLEVEL *w_current)
+o_select_move_to_place_list(GschemToplevel *w_current)
 {
-  TOPLEVEL *toplevel = w_current->toplevel;
+  TOPLEVEL *toplevel = gschem_toplevel_get_toplevel (w_current);
   GList *selection;
   GList *selection_copy;
 
