@@ -1,7 +1,7 @@
 /* gEDA - GPL Electronic Design Automation
  * gschem - gEDA Schematic Capture
  * Copyright (C) 1998-2010 Ales Hvezda
- * Copyright (C) 1998-2010 gEDA Contributors (see ChangeLog for details)
+ * Copyright (C) 1998-2019 gEDA Contributors (see ChangeLog for details)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,10 +27,6 @@
 
 #include "gschem.h"
 
-#ifdef HAVE_LIBDMALLOC
-#include <dmalloc.h>
-#endif
-
 COLOR display_colors[MAX_COLORS];
 COLOR display_outline_colors[MAX_COLORS];
 
@@ -150,6 +146,8 @@ void x_color_allocate (void)
       gdk_outline_colors[i] = NULL;
     }
   }
+
+  x_colorcb_update_store ();
 }
 
 /*! \todo Finish function documentation!!!
@@ -168,23 +166,6 @@ GdkColor *x_get_color(int color)
   }
 }
 
-/*! \todo Finish function documentation!!!
- *  \brief
- *  \par Function Documentation
- *
- *  \todo this has to change... to the right code
- */
-GdkColor *x_get_darkcolor(int color)
-{
-  if ((color < 0) || (color >= MAX_COLORS)
-      || (gdk_outline_colors[color] == NULL)) {
-    g_warning (_("Tried to get an invalid color: %d\n"), color);
-    return(&white);
-  } else {
-    return(gdk_outline_colors[color]);
-  }
-}
-
 /*! \todo Finish function documentation!!!
  *  \brief
  *  \par Function Documentation
@@ -201,46 +182,6 @@ COLOR *x_color_lookup (int color)
   }
 }
 
-/*! \todo Finish function documentation!!!
- *  \brief
- *  \par Function Documentation
- *
- *  \todo this has to change... to the right code
- */
-COLOR *x_color_lookup_dark (int color)
-{
-  if (color < 0 || color >= MAX_COLORS ||
-      !display_outline_colors[color].enabled) {
-    g_warning (_("Tried to get an invalid outline color: %d\n"), color);
-    return &display_outline_colors[DEFAULT_COLOR];
-  } else {
-    return &display_outline_colors[color];
-  }
-
-}
-
-/*! \todo Finish function documentation!!!
- *  \brief
- *  \par Function Documentation
- *
- */
-gchar *x_color_get_name(int index)
-{
-  COLOR c;
-
-  if ((index >= MAX_COLORS) || (index < 0)) {
-    return(NULL);
-  }
-
-  if (display_colors[index].enabled) {
-    c = display_colors[index];
-    return s_color_rgba_encode (c.r, c.g, c.b, c.a);
-  }
-
-  /* didn't find a color, but there still might be more */
-  return(NULL);
-}
-
 gboolean
 x_color_display_enabled (int index)
 {
