@@ -1,7 +1,7 @@
 /* gEDA - GPL Electronic Design Automation
  * libgeda - gEDA's library
  * Copyright (C) 1998-2010 Ales Hvezda
- * Copyright (C) 1998-2010 gEDA Contributors (see ChangeLog for details)
+ * Copyright (C) 1998-2019 gEDA Contributors (see ChangeLog for details)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,10 +31,6 @@
 
 #include "libgeda_priv.h"
 
-#ifdef HAVE_LIBDMALLOC
-#include <dmalloc.h>
-#endif
-
 /*! \brief Get the file header string.
  *  \par Function Description
  *  This function simply returns the DATE_VERSION and
@@ -58,20 +54,17 @@ const gchar *o_file_format_header()
  *  This function saves a whole schematic into a buffer in libgeda
  *  format. The buffer should be freed when no longer needed.
  *
- *  \param [in] toplevel    The current TOPLEVEL.
  *  \param [in] object_list The head of a GList of OBJECTs to save.
  *  \returns a buffer containing schematic data or NULL on failure.
  */
-gchar *o_save_buffer (TOPLEVEL *toplevel, const GList *object_list)
+gchar *o_save_buffer (const GList *object_list)
 {
   GString *acc;
   gchar *buffer;
 
-  if (toplevel == NULL) return NULL;
-
   acc = g_string_new (o_file_format_header());
 
-  buffer = o_save_objects (toplevel, object_list, FALSE);
+  buffer = o_save_objects (object_list, FALSE);
   g_string_append (acc, buffer);
   g_free (buffer);
 
@@ -89,12 +82,11 @@ gchar *o_save_buffer (TOPLEVEL *toplevel, const GList *object_list)
  *  we recurse for saving out those attributes, the function must be called
  *  with save_attribs passed as TRUE.
  *
- *  \param [in] toplevel      A TOPLEVEL structure.
  *  \param [in] object_list   The head of a GList of objects to save.
  *  \param [in] save_attribs  Should attribute objects encounterd be saved?
  *  \returns a buffer containing schematic data or NULL on failure.
  */
-gchar *o_save_objects (TOPLEVEL *toplevel, const GList *object_list, gboolean save_attribs)
+gchar *o_save_objects (const GList *object_list, gboolean save_attribs)
 {
   OBJECT *o_current;
   const GList *iter;
@@ -114,27 +106,27 @@ gchar *o_save_objects (TOPLEVEL *toplevel, const GList *object_list, gboolean sa
       switch (o_current->type) {
 
         case(OBJ_LINE):
-          out = o_line_save(toplevel, o_current);
+          out = o_line_save(o_current);
           break;
 
         case(OBJ_NET):
-          out = o_net_save(toplevel, o_current);
+          out = o_net_save(o_current);
           break;
 
         case(OBJ_BUS):
-          out = o_bus_save(toplevel, o_current);
+          out = o_bus_save(o_current);
           break;
 
         case(OBJ_BOX):
-          out = o_box_save(toplevel, o_current);
+          out = o_box_save(o_current);
           break;
 
         case(OBJ_CIRCLE):
-          out = o_circle_save(toplevel, o_current);
+          out = o_circle_save(o_current);
           break;
 
         case(OBJ_COMPLEX):
-          out = o_complex_save(toplevel, o_current);
+          out = o_complex_save(o_current);
           g_string_append_printf(acc, "%s\n", out);
           already_wrote = TRUE;
           g_free(out); /* need to free here because of the above flag */
@@ -142,7 +134,7 @@ gchar *o_save_objects (TOPLEVEL *toplevel, const GList *object_list, gboolean sa
           if (o_complex_is_embedded(o_current)) {
             g_string_append(acc, "[\n");
 
-            out = o_save_objects(toplevel, o_current->complex->prim_objs, FALSE);
+            out = o_save_objects(o_current->complex->prim_objs, FALSE);
             g_string_append (acc, out);
             g_free(out);
 
@@ -151,27 +143,27 @@ gchar *o_save_objects (TOPLEVEL *toplevel, const GList *object_list, gboolean sa
           break;
 
         case(OBJ_PLACEHOLDER):  /* new type by SDB 1.20.2005 */
-          out = o_complex_save(toplevel, o_current);
+          out = o_complex_save(o_current);
           break;
 
         case(OBJ_TEXT):
-          out = o_text_save(toplevel, o_current);
+          out = o_text_save(o_current);
           break;
 
         case(OBJ_PATH):
-          out = o_path_save(toplevel, o_current);
+          out = o_path_save(o_current);
           break;
 
         case(OBJ_PIN):
-          out = o_pin_save(toplevel, o_current);
+          out = o_pin_save(o_current);
           break;
 
         case(OBJ_ARC):
-          out = o_arc_save(toplevel, o_current);
+          out = o_arc_save(o_current);
           break;
 
         case(OBJ_PICTURE):
-          out = o_picture_save(toplevel, o_current);
+          out = o_picture_save(o_current);
           break;
 
         default:
@@ -198,7 +190,7 @@ gchar *o_save_objects (TOPLEVEL *toplevel, const GList *object_list, gboolean sa
       if (o_current->attribs != NULL) {
         g_string_append (acc, "{\n");
 
-        out = o_save_objects (toplevel, o_current->attribs, TRUE);
+        out = o_save_objects (o_current->attribs, TRUE);
         g_string_append (acc, out);
         g_free(out);
 
@@ -239,7 +231,7 @@ int o_save (TOPLEVEL *toplevel, const GList *object_list,
     return 0;      
   }
 
-  buffer = o_save_buffer (toplevel, object_list);
+  buffer = o_save_buffer (object_list);
   if (!g_file_set_contents (filename, buffer, strlen(buffer), err)) {
     g_free (buffer);
     return 0;
@@ -281,7 +273,6 @@ GList *o_read_buffer (TOPLEVEL *toplevel, GList *object_list,
   GList *iter;
   unsigned int release_ver = 0;
   unsigned int fileformat_ver = 0;
-  unsigned int current_fileformat_ver = FILEFORMAT_VERSION;
   int found_pin = 0;
   OBJECT* last_complex = NULL;
   int itemsread = 0;
@@ -290,6 +281,13 @@ GList *o_read_buffer (TOPLEVEL *toplevel, GList *object_list,
 
   g_return_val_if_fail ((buffer != NULL), NULL);
 
+  /* Check the buffer is valid UTF-8 */
+  if (!g_utf8_validate (buffer, (size < 0) ? -1 : size, NULL)) {
+    g_set_error (err, EDA_ERROR, EDA_ERROR_UNKNOWN_ENCODING,
+                 _("Schematic data was not valid UTF-8"));
+    return NULL;
+  }
+
   tb = s_textbuffer_new (buffer, size);
 
   while (1) {
@@ -394,12 +392,10 @@ GList *o_read_buffer (TOPLEVEL *toplevel, GList *object_list,
         /* first is the fp */
         /* 2nd is the object to get the attributes */
         if (new_obj != NULL) {
-          o_attrib_freeze_hooks (toplevel, new_obj);
           new_attrs_list = o_read_attribs (toplevel, new_obj, tb, release_ver, fileformat_ver, err);
           if (new_attrs_list == NULL)
             goto error;
           new_object_list = g_list_concat (new_attrs_list, new_object_list);
-          o_attrib_thaw_hooks (toplevel, new_obj);
 
           /* by now we have finished reading all the attributes */
           /* did we just finish attaching to a complex object? */
@@ -464,7 +460,7 @@ GList *o_read_buffer (TOPLEVEL *toplevel, GList *object_list,
             tmp->parent = new_obj;
           }
 
-          o_recalc_single_object (toplevel, new_obj);
+          new_obj->w_bounds_valid_for = NULL;
 
           embedded_level--;
         } else {
@@ -531,7 +527,7 @@ GList *o_read_buffer (TOPLEVEL *toplevel, GList *object_list,
     }
   }
 
-  tb = s_textbuffer_free(tb);
+  s_textbuffer_free(tb);
 
   new_object_list = g_list_reverse(new_object_list);
   object_list = g_list_concat (object_list, new_object_list);
@@ -573,42 +569,3 @@ GList *o_read (TOPLEVEL *toplevel, GList *object_list, char *filename,
   g_free (buffer);
   return result;
 }
-
-/*! \brief Scale a set of lines.
- *  \par Function Description
- *  This function takes a list of lines and scales them
- *  by the values of x_scale and y_scale.
- *
- *  \param [in] toplevel  The current TOPLEVEL object.
- *  \param [in,out]  list  The list with lines to scale.
- *  \param [in]   x_scale  The x scale value for the lines.
- *  \param [in]   y_scale  The y scale value for the lines.
- *
- *  \todo this really doesn't belong here. you need more of a core routine
- *        first. yes.. this is the core routine, just strip out the drawing
- *        stuff
- *        move it to o_complex_scale
- */
-void o_scale (TOPLEVEL *toplevel, GList *list, int x_scale, int y_scale)
-{
-  OBJECT *o_current;
-  GList *iter;
-
-  /* this is okay if you just hit scale and have nothing selected */
-  if (list == NULL) {
-    return;
-  }
-
-  iter = list;
-  while (iter != NULL) {
-    o_current = (OBJECT *)iter->data;
-    switch(o_current->type) {
-      case(OBJ_LINE):
-        o_line_scale_world(toplevel, x_scale, y_scale, o_current);
-        break;
-    }
-    iter = g_list_next (iter);
-  }
-}
-
-
