@@ -1,7 +1,7 @@
 /* gEDA - GPL Electronic Design Automation
  * libgeda - gEDA's library
  * Copyright (C) 1998-2010 Ales Hvezda
- * Copyright (C) 1998-2010 gEDA Contributors (see ChangeLog for details)
+ * Copyright (C) 1998-2019 gEDA Contributors (see ChangeLog for details)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,7 +41,6 @@
  */
 
 #include <config.h>
-#include <missing.h>
 
 #include <stdio.h>
 #ifdef HAVE_STRING_H
@@ -51,10 +50,6 @@
 
 #include "libgeda_priv.h"
 
-#ifdef HAVE_LIBDMALLOC
-#include <dmalloc.h>
-#endif
-
 
 /*! \brief Add an attribute to an existing attribute list.
  *  \par Function Description
@@ -70,8 +65,6 @@ void o_attrib_add(TOPLEVEL *toplevel, OBJECT *object, OBJECT *item)
   /* Add link from item to attrib listing */
   item->attached_to = object;
   object->attribs = g_list_append (object->attribs, item);
-
-  o_attrib_emit_attribs_changed (toplevel, object);
 }
 
 
@@ -115,7 +108,7 @@ void o_attrib_attach (TOPLEVEL *toplevel, OBJECT *attrib, OBJECT *object,
 
   /* is the object already part of the list ? */
   if (g_list_find (object->attribs, attrib)) {
-    g_warning ("Attribute [%s] already attached\n", attrib->text->string);
+    g_warning (_("Attribute [%s] already attached\n"), attrib->text->string);
     return;
   }
 
@@ -168,21 +161,16 @@ void o_attrib_detach_all(TOPLEVEL *toplevel, OBJECT *object)
   OBJECT *a_current;
   GList *a_iter;
 
-  o_attrib_freeze_hooks (toplevel, object);
-
   for (a_iter = object->attribs; a_iter != NULL;
        a_iter = g_list_next (a_iter)) {
     a_current = a_iter->data;
 
     a_current->attached_to = NULL;
     o_set_color (toplevel, a_current, DETACHED_ATTRIBUTE_COLOR);
-    o_attrib_emit_attribs_changed (toplevel, object);
   }
 
   g_list_free (object->attribs);
   object->attribs = NULL;
-
-  o_attrib_thaw_hooks (toplevel, object);
 }
 
 /*! \brief Print all attributes to a Postscript document.
@@ -221,16 +209,11 @@ void o_attrib_print(GList *attributes)
  */
 void o_attrib_remove(TOPLEVEL *toplevel, GList **list, OBJECT *remove)
 {
-  OBJECT *attached_to;
-
   g_return_if_fail (remove != NULL);
 
-  attached_to = remove->attached_to;
   remove->attached_to = NULL;
 
   *list = g_list_remove (*list, remove);
-
-  o_attrib_emit_attribs_changed (toplevel, attached_to);
 }
 
 /*! \brief Read attributes from a buffer.
@@ -700,65 +683,3 @@ int o_attrib_is_inherited (OBJECT *attrib)
   return (attrib->attached_to == NULL &&
           attrib->parent != NULL);
 }
-
-
-typedef struct {
-  AttribsChangedFunc func;
-  void *data;
-} AttribsChangedHook;
-
-
-void o_attrib_append_attribs_changed_hook (TOPLEVEL *toplevel,
-                                           AttribsChangedFunc func,
-                                           void *data)
-{
-  AttribsChangedHook *new_hook;
-
-  new_hook = g_new0 (AttribsChangedHook, 1);
-  new_hook->func = func;
-  new_hook->data = data;
-
-  toplevel->attribs_changed_hooks =
-    g_list_append (toplevel->attribs_changed_hooks, new_hook);
-}
-
-
-static void call_attribs_changed_hook (gpointer data, gpointer user_data)
-{
-  AttribsChangedHook *hook = data;
-  OBJECT *object = user_data;
-
-  hook->func (hook->data, object);
-}
-
-
-void o_attrib_emit_attribs_changed (TOPLEVEL *toplevel, OBJECT *object)
-{
-  if (object->attrib_notify_freeze_count > 0) {
-    object->attrib_notify_pending = 1;
-    return;
-  }
-
-//  printf ("The attributes of object %p have changed\n", object);
-
-  object->attrib_notify_pending = 0;
-
-  g_list_foreach (toplevel->attribs_changed_hooks,
-                  call_attribs_changed_hook, object);
-}
-
-void o_attrib_freeze_hooks (TOPLEVEL *toplevel, OBJECT *object)
-{
-  object->attrib_notify_freeze_count ++;
-}
-
-void o_attrib_thaw_hooks (TOPLEVEL *toplevel, OBJECT *object)
-{
-  g_return_if_fail (object->attrib_notify_freeze_count > 0);
-
-  object->attrib_notify_freeze_count --;
-
-  if (object->attrib_notify_freeze_count == 0 &&
-      object->attrib_notify_pending)
-    o_attrib_emit_attribs_changed (toplevel, object);
-}
