@@ -1,7 +1,7 @@
 /* gEDA - GPL Electronic Design Automation
  * libgeda - gEDA's library
  * Copyright (C) 1998-2010 Ales Hvezda
- * Copyright (C) 1998-2010 gEDA Contributors (see ChangeLog for details)
+ * Copyright (C) 1998-2019 gEDA Contributors (see ChangeLog for details)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,6 @@
 #endif
 #include <math.h>
 
-#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gio/gio.h>
 
 #include "libgeda_priv.h"
@@ -48,11 +47,11 @@
  *  \return A pointer to the new picture object, or NULL on error.
  */
 OBJECT *o_picture_read (TOPLEVEL *toplevel,
-		       const char *first_line,
-		       TextBuffer *tb,
-		       unsigned int release_ver,
-               unsigned int fileformat_ver,
-               GError **err)
+                        const char *first_line,
+                        TextBuffer *tb,
+                        unsigned int release_ver,
+                        unsigned int fileformat_ver,
+                        GError **err)
 {
   OBJECT *new_obj;
   int x1, y1;
@@ -66,8 +65,8 @@ OBJECT *o_picture_read (TOPLEVEL *toplevel,
   guint file_length = 0;
 
   num_conv = sscanf(first_line, "%c %d %d %d %d %d %d %d\n",
-	 &type, &x1, &y1, &width, &height, &angle, &mirrored, &embedded);
-  
+                    &type, &x1, &y1, &width, &height, &angle, &mirrored, &embedded);
+
   if (num_conv != 8) {
     g_set_error(err, EDA_ERROR, EDA_ERROR_PARSE, _("Failed to parse picture definition"));
     return NULL;
@@ -80,20 +79,20 @@ OBJECT *o_picture_read (TOPLEVEL *toplevel,
 
   if ( (mirrored > 1) || (mirrored < 0)) {
     s_log_message(_("Found a picture with a wrong 'mirrored' parameter: %d.\n"),
-	    mirrored);
+                  mirrored);
     s_log_message(_("Setting mirrored to 0\n"));
     mirrored = 0;
   }
 
   if ( (embedded > 1) || (embedded < 0)) {
     s_log_message(_("Found a picture with a wrong 'embedded' parameter: %d.\n"),
-	    embedded);
+                  embedded);
     s_log_message(_("Setting embedded to 0\n"));
     embedded = 0;
   }
 
   switch(angle) {
-	
+
     case(0):
     case(90):
     case(180):
@@ -109,7 +108,7 @@ OBJECT *o_picture_read (TOPLEVEL *toplevel,
   }
 
   filename = g_strdup(s_textbuffer_next_line(tb));
-  filename = remove_last_nl(filename);	
+  filename = remove_last_nl(filename);
 
   /* Handle empty filenames */
   if (strlen (filename) == 0) {
@@ -128,7 +127,7 @@ OBJECT *o_picture_read (TOPLEVEL *toplevel,
       line = s_textbuffer_next_line(tb);
       if (line == NULL) break;
 
-      if (g_strcasecmp(line, ".\n") != 0) {
+      if (strcmp (line, ".\n") != 0) {
         encoded_picture = g_string_append (encoded_picture, line);
       } else {
         finished = 1;
@@ -150,7 +149,7 @@ OBJECT *o_picture_read (TOPLEVEL *toplevel,
       embedded = 0;
     }
   }
-  
+
   /* create the picture */
   /* The picture is described by its upper left and lower right corner */
   new_obj = o_picture_new (toplevel, file_content, file_length, filename,
@@ -169,7 +168,6 @@ OBJECT *o_picture_read (TOPLEVEL *toplevel,
  *  This function formats a string in the buffer <B>*buff</B> to describe
  *  the picture object <B>*object</B>.
  *
- *  \param [in] toplevel  a TOPLEVEL structure
  *  \param [in] object  Picture OBJECT to create string from.
  *  \return A pointer to the picture OBJECT character string.
  *
@@ -177,7 +175,7 @@ OBJECT *o_picture_read (TOPLEVEL *toplevel,
  *  Caller must g_free returned character string.
  *
  */
-char *o_picture_save(TOPLEVEL *toplevel, OBJECT *object)
+char *o_picture_save(OBJECT *object)
 {
   int width, height, x1, y1;
   gchar *encoded_picture=NULL;
@@ -186,7 +184,7 @@ char *o_picture_save(TOPLEVEL *toplevel, OBJECT *object)
   const gchar *filename = NULL;
 
   /* calculate the width and height of the box */
-  width  = abs(object->picture->lower_x - object->picture->upper_x); 
+  width  = abs(object->picture->lower_x - object->picture->upper_x);
   height = abs(object->picture->upper_y - object->picture->lower_y);
 
   /* calculate the lower left corner of the box */
@@ -198,7 +196,7 @@ char *o_picture_save(TOPLEVEL *toplevel, OBJECT *object)
 #endif
 
   /* Encode the picture if it's embedded */
-  if (o_picture_is_embedded (toplevel, object)) {
+  if (o_picture_is_embedded (object)) {
     encoded_picture =
       s_encoding_base64_encode( (char *)object->picture->file_content,
                                 object->picture->file_length,
@@ -210,31 +208,31 @@ char *o_picture_save(TOPLEVEL *toplevel, OBJECT *object)
   }
 
   /* Cope with null filename */
-  filename = o_picture_get_filename (toplevel, object);
+  filename = o_picture_get_filename (object);
   if (filename == NULL) filename = "";
 
-  if (o_picture_is_embedded (toplevel, object) &&
+  if (o_picture_is_embedded (object) &&
       encoded_picture != NULL) {
-    out = g_strdup_printf("%c %d %d %d %d %d %c %c\n%s\n%s\n%s", 
-			  object->type,
-			  x1, y1, width, height,
-			  object->picture->angle,
-			  /* Convert the (0,1) chars to ASCII */
-			  (object->picture->mirrored)+0x30, 
-			  '1', 
-			  filename,
-			  encoded_picture,
-			  ".");
+    out = g_strdup_printf("%c %d %d %d %d %d %c %c\n%s\n%s\n%s",
+                          object->type,
+                          x1, y1, width, height,
+                          object->picture->angle,
+                          /* Convert the (0,1) chars to ASCII */
+                          (object->picture->mirrored)+0x30,
+                          '1',
+                          filename,
+                          encoded_picture,
+                          ".");
   }
   else {
-    out = g_strdup_printf("%c %d %d %d %d %d %c %c\n%s", 
-			  object->type,
-			  x1, y1, width, height,
-			  object->picture->angle,
-			  /* Convert the (0,1) chars to ASCII */
-			  (object->picture->mirrored)+0x30, 
-			  '0', 
-			  filename);
+    out = g_strdup_printf("%c %d %d %d %d %d %c %c\n%s",
+                          object->type,
+                          x1, y1, width, height,
+                          object->picture->angle,
+                          /* Convert the (0,1) chars to ASCII */
+                          (object->picture->mirrored)+0x30,
+                          '0',
+                          filename);
   }
   g_free(encoded_picture);
 
@@ -247,7 +245,7 @@ char *o_picture_save(TOPLEVEL *toplevel, OBJECT *object)
  *  This function creates a new object representing a picture.
  *
  *  The picture is described by its upper left corner (\a x1, \a y1)
- *  and its lower right corner (\a x2, \ay2).  The \a type parameter
+ *  and its lower right corner (\a x2, \a y2).  The \a type parameter
  *  must be equal to #OBJ_PICTURE.
  *
  *  If \a file_content is non-NULL, it must be a pointer to a buffer
@@ -295,7 +293,7 @@ OBJECT *o_picture_new (TOPLEVEL *toplevel,
   picture->file_content = NULL;
   picture->file_length = 0;
 
-  picture->ratio = abs ((double) (x1 - x2) / (y1 - y2));
+  picture->ratio = fabs ((double) (x1 - x2) / (y1 - y2));
   picture->filename = g_strdup (filename);
   picture->angle = angle;
   picture->mirrored = mirrored;
@@ -321,41 +319,17 @@ OBJECT *o_picture_new (TOPLEVEL *toplevel,
       s_log_message (_("Failed to load image from [%s]: %s\n"),
                      filename, error->message);
       g_error_free (error);
+      /* picture not found; try to open a fall back pixbuf */
+      picture->pixbuf = o_picture_get_fallback_pixbuf(toplevel);
     }
   }
 
   /* compute the bounding picture */
-  o_picture_recalc(toplevel, new_node);
+  new_node->w_bounds_valid_for = NULL;
 
   return new_node;
 }
 
-/*! \brief Recalculate picture bounding box.
- *  \par Function Description
- *  This function recalculates the bounding box of the <B>o_current</B>
- *  parameter picture object.
- *
- *  \param [in] toplevel      The TOPLEVEL object.
- *  \param [in,out] o_current  Picture OBJECT to be recalculated.
- */
-void o_picture_recalc(TOPLEVEL *toplevel, OBJECT *o_current)
-{
-  int left, top, right, bottom;
-
-  if (o_current->picture == NULL) {
-    return;
-  }
-
-  /* update the bounding picture - world units */
-  world_get_picture_bounds(toplevel, o_current,
-		     &left, &top, &right, &bottom);
-  o_current->w_left   = left;
-  o_current->w_top    = top;
-  o_current->w_right  = right;
-  o_current->w_bottom = bottom;
-  o_current->w_bounds_valid = TRUE;
-}
-
 /*! \brief Get picture bounding rectangle in WORLD coordinates.
  *  \par Function Description
  *  This function sets the <B>left</B>, <B>top</B>, <B>right</B> and
@@ -383,20 +357,28 @@ void world_get_picture_bounds(TOPLEVEL *toplevel, OBJECT *object,
  *  \par Function Description
  *  This function gets the position of the bottom left point of a picture object.
  *
- *  \param [in] toplevel The toplevel environment.
+ *  \param [in] object   The object to get the position.
  *  \param [out] x       pointer to the x-position
  *  \param [out] y       pointer to the y-position
- *  \param [in] object   The object to get the position.
  *  \return TRUE if successfully determined the position, FALSE otherwise
  */
-gboolean o_picture_get_position (TOPLEVEL *toplevel, gint *x, gint *y,
-                              OBJECT *object)
+gboolean o_picture_get_position (OBJECT *object, gint *x, gint *y)
 {
-  *x = min(object->picture->lower_x, object->picture->upper_x);
-  *y = min(object->picture->lower_y, object->picture->upper_y);
+  g_return_val_if_fail (object != NULL, FALSE);
+  g_return_val_if_fail (object->type == OBJ_PICTURE, FALSE);
+  g_return_val_if_fail (object->picture != NULL, FALSE);
+
+  if (x != NULL) {
+    *x = min (object->picture->lower_x, object->picture->upper_x);
+  }
+
+  if (y != NULL) {
+    *y = min (object->picture->lower_y, object->picture->upper_y);
+  }
+
   return TRUE;
 }
-                 
+
 
 /*! \brief Get the width/height ratio of an image.
  * \par Function Description
@@ -454,7 +436,7 @@ o_picture_get_ratio (TOPLEVEL *toplevel, OBJECT *object)
  *  </DL>
  */
 void o_picture_modify(TOPLEVEL *toplevel, OBJECT *object,
-		      int x, int y, int whichone)
+                      int x, int y, int whichone)
 {
   int tmp;
   double ratio = o_picture_get_ratio (toplevel, object);
@@ -467,57 +449,57 @@ void o_picture_modify(TOPLEVEL *toplevel, OBJECT *object,
       object->picture->upper_x = x;
       tmp = abs(object->picture->upper_x - object->picture->lower_x) / ratio;
       if (y < object->picture->lower_y) {
-	tmp = -tmp;
+        tmp = -tmp;
       }
       object->picture->upper_y = object->picture->lower_y + tmp;
       break;
-			
+
     case PICTURE_LOWER_LEFT:
       object->picture->upper_x = x;
       tmp = abs(object->picture->upper_x - object->picture->lower_x) / ratio;
       if (y > object->picture->upper_y) {
-	tmp = -tmp;
+        tmp = -tmp;
       }
       object->picture->lower_y = object->picture->upper_y - tmp;
       break;
-      
+
     case PICTURE_UPPER_RIGHT:
       object->picture->lower_x = x;
       tmp = abs(object->picture->upper_x - object->picture->lower_x) / ratio;
       if (y < object->picture->lower_y) {
-	tmp = -tmp;
+        tmp = -tmp;
       }
       object->picture->upper_y = object->picture->lower_y + tmp;
       break;
-      
+
     case PICTURE_LOWER_RIGHT:
       object->picture->lower_x = x;
       tmp = abs(object->picture->upper_x - object->picture->lower_x) / ratio;
       if (y > object->picture->upper_y) {
-	tmp = -tmp;
+        tmp = -tmp;
       }
       object->picture->lower_y = object->picture->upper_y - tmp;
       break;
-      
+
     default:
       return;
   }
-  
+
   /* need to update the upper left and lower right corners */
   if(object->picture->upper_x > object->picture->lower_x) {
     tmp                      = object->picture->upper_x;
     object->picture->upper_x = object->picture->lower_x;
     object->picture->lower_x = tmp;
   }
-  
+
   if(object->picture->upper_y < object->picture->lower_y) {
     tmp                      = object->picture->upper_y;
     object->picture->upper_y = object->picture->lower_y;
     object->picture->lower_y = tmp;
   }
-	
+
   /* recalculate the screen coords and the boundings */
-  o_picture_recalc(toplevel, object);
+  object->w_bounds_valid_for = NULL;
   o_emit_change_notify (toplevel, object);
 }
 
@@ -548,12 +530,12 @@ o_picture_modify_all (TOPLEVEL *toplevel, OBJECT *object,
   object->picture->upper_y = (y1 > y2) ? y1 : y2;
 
   /* recalculate the world coords and bounds */
-  o_box_recalc(toplevel, object);
+  object->w_bounds_valid_for = NULL;
   o_emit_change_notify (toplevel, object);
 }
 
 /*! \brief Rotate picture OBJECT using WORLD coordinates.
- *  \par Function Description 
+ *  \par Function Description
  *  This function rotates the picture described by <B>*object</B> around
  *  the (<B>world_centerx</B>, <B>world_centery</B>) point by <B>angle</B>
  *  degrees.
@@ -568,20 +550,24 @@ o_picture_modify_all (TOPLEVEL *toplevel, OBJECT *object,
  *  \param [in,out]  object         Picture OBJECT to rotate.
  */
 void o_picture_rotate_world(TOPLEVEL *toplevel,
-			    int world_centerx, int world_centery, int angle,
-			    OBJECT *object)
+                            int world_centerx, int world_centery, int angle,
+                            OBJECT *object)
 {
   int newx1, newy1;
   int newx2, newy2;
-  
+
+  g_return_if_fail (object != NULL);
+  g_return_if_fail (object->picture != NULL);
+  g_return_if_fail (object->type == OBJ_PICTURE);
+
   /* Only 90 degree multiple and positive angles are allowed. */
   /* angle must be positive */
   if(angle < 0) angle = -angle;
   /* angle must be a 90 multiple or no rotation performed */
   if((angle % 90) != 0) return;
-  
+
   object->picture->angle = ( object->picture->angle + angle ) % 360;
-	
+
   /* The center of rotation (<B>world_centerx</B>, <B>world_centery</B>) is
    * translated to the origin. The rotation of the upper left and lower
    * right corner are then performed. Finally, the rotated picture is
@@ -592,30 +578,30 @@ void o_picture_rotate_world(TOPLEVEL *toplevel,
   object->picture->upper_y -= world_centery;
   object->picture->lower_x -= world_centerx;
   object->picture->lower_y -= world_centery;
-  
+
   /* rotate the upper left corner of the picture */
   rotate_point_90(object->picture->upper_x, object->picture->upper_y, angle,
-		  &newx1, &newy1);
-  
+                  &newx1, &newy1);
+
   /* rotate the lower left corner of the picture */
   rotate_point_90(object->picture->lower_x, object->picture->lower_y, angle,
-		  &newx2, &newy2);
-  
+                  &newx2, &newy2);
+
   /* reorder the corners after rotation */
   object->picture->upper_x = min(newx1,newx2);
   object->picture->upper_y = max(newy1,newy2);
   object->picture->lower_x = max(newx1,newx2);
   object->picture->lower_y = min(newy1,newy2);
-  
+
   /* translate object back to normal position */
   object->picture->upper_x += world_centerx;
   object->picture->upper_y += world_centery;
   object->picture->lower_x += world_centerx;
   object->picture->lower_y += world_centery;
-  
+
   /* recalc boundings and screen coords */
-  o_picture_recalc(toplevel, object);
-	
+  object->w_bounds_valid_for = NULL;
+
 }
 
 /*! \brief Mirror a picture using WORLD coordinates.
@@ -632,12 +618,16 @@ void o_picture_rotate_world(TOPLEVEL *toplevel,
  *  \param [in,out] object         Picture OBJECT to mirror.
  */
 void o_picture_mirror_world(TOPLEVEL *toplevel,
-			    int world_centerx, int world_centery,
-			    OBJECT *object)
+                            int world_centerx, int world_centery,
+                            OBJECT *object)
 {
   int newx1, newy1;
   int newx2, newy2;
 
+  g_return_if_fail (object != NULL);
+  g_return_if_fail (object->picture != NULL);
+  g_return_if_fail (object->type == OBJ_PICTURE);
+
   /* Set info in object. Sometimes it's necessary to change the
    * rotation angle as well as the mirror flag. */
   object->picture->mirrored = !object->picture->mirrored;
@@ -675,8 +665,8 @@ void o_picture_mirror_world(TOPLEVEL *toplevel,
   object->picture->lower_y += world_centery;
 
   /* recalc boundings and screen coords */
-  o_picture_recalc(toplevel, object);
-  
+  object->w_bounds_valid_for = NULL;
+
 }
 
 /*! \brief Translate a picture position in WORLD coordinates by a delta.
@@ -684,22 +674,24 @@ void o_picture_mirror_world(TOPLEVEL *toplevel,
  *  This function applies a translation of (<B>x1</B>,<B>y1</B>) to the picture
  *  described by <B>*object</B>. <B>x1</B> and <B>y1</B> are in world units.
  *
- *  \param [in]     toplevel  The TOPLEVEL object.
+ *  \param [in,out] object     Picture OBJECT to translate.
  *  \param [in]     dx         x distance to move.
  *  \param [in]     dy         y distance to move.
- *  \param [in,out] object     Picture OBJECT to translate.
  */
-void o_picture_translate_world(TOPLEVEL *toplevel,
-			       int dx, int dy, OBJECT *object)
+void o_picture_translate_world(OBJECT *object, int dx, int dy)
 {
+  g_return_if_fail (object != NULL);
+  g_return_if_fail (object->picture != NULL);
+  g_return_if_fail (object->type == OBJ_PICTURE);
+
   /* Do world coords */
   object->picture->upper_x = object->picture->upper_x + dx;
   object->picture->upper_y = object->picture->upper_y + dy;
   object->picture->lower_x = object->picture->lower_x + dx;
   object->picture->lower_y = object->picture->lower_y + dy;
-  
+
   /* recalc the screen coords and the bounding picture */
-  o_picture_recalc(toplevel, object);
+  object->w_bounds_valid_for = NULL;
 }
 
 /*! \brief Create a copy of a picture.
@@ -749,195 +741,11 @@ OBJECT *o_picture_copy(TOPLEVEL *toplevel, OBJECT *object)
   picture->pixbuf = o_picture_get_pixbuf (toplevel, object);
 
   /* compute the bounding picture */
-  o_picture_recalc(toplevel, new_node);
+  new_node->w_bounds_valid_for = NULL;
 
   return new_node;
 }
 
-
-/*! \brief Get RGB data from image.
- *  \par Function Description
- *  This function returns the RGB data of the given image.
- *  Function taken from the DIA source code (http://www.gnome.org/projects/dia)
- *  and licensed under the GNU GPL version 2.
- *
- *  \param [in] image  GdkPixbuf image to read RGB data from.
- *  \return Array of rgb data from image.
- *
- *  \note
- *  Caller must g_free returned guint8 array.
- */
-static guint8 *
-o_picture_rgb_data(GdkPixbuf *image)
-{
-  int width = gdk_pixbuf_get_width(image);
-  int height = gdk_pixbuf_get_height(image);
-  int rowstride = gdk_pixbuf_get_rowstride(image);
-  int size = height*rowstride;
-  guint8 *rgb_pixels = g_malloc(size);
-
-  if (gdk_pixbuf_get_has_alpha(image)) {
-    guint8 *pixels = gdk_pixbuf_get_pixels(image);
-    int i, j;
-    for (i = 0; i < height; i++) {
-      for (j = 0; j < width; j++) {
-	rgb_pixels[i*rowstride+j*3] = pixels[i*rowstride+j*4];
-	rgb_pixels[i*rowstride+j*3+1] = pixels[i*rowstride+j*4+1];
-	rgb_pixels[i*rowstride+j*3+2] = pixels[i*rowstride+j*4+2];
-      }
-    }
-    return rgb_pixels;
-  } else {
-    guint8 *pixels = gdk_pixbuf_get_pixels(image);
-
-    g_memmove(rgb_pixels, pixels, height*rowstride);
-    return rgb_pixels;
-  }
-}
-
-/*! \brief Get mask data from image.
- *  \par Function Description
- *  This function returns the mask data of the given image.
- *  Function taken from the DIA source code (http://www.gnome.org/projects/dia)
- *  and licensed under the GNU GPL version 2.
- *
- *  \param [in] image  GdkPixbuf image to get mask data from.
- *  \return Array of mask data from image.
- *
- *  \note
- *  Caller must g_free returned guint8 array.
- */
-static guint8 *
-o_picture_mask_data(GdkPixbuf *image)
-{
-  guint8 *pixels;
-  guint8 *mask;
-  int i, size;
-
-  if (!gdk_pixbuf_get_has_alpha(image)) {
-    return NULL;
-  }
-  
-  pixels = gdk_pixbuf_get_pixels(image);
-
-  size = gdk_pixbuf_get_width(image)*
-    gdk_pixbuf_get_height(image);
-
-  mask = g_malloc(size);
-
-  /* Pick every fourth byte (the alpha channel) into mask */
-  for (i = 0; i < size; i++)
-    mask[i] = pixels[i*4+3];
-
-  return mask;
-}
-
-/*! \brief Print picture to Postscript document.
- *  \par Function Description
- *  This function prints a picture object. The picture is defined by the
- *  coordinates of its upper left corner in (<B>x</B>,<B>y</B>) and its width
- *  and height given by the <B>width</B> and <B>height</B> parameters. 
- *
- *  If the picture object was unable to be loaded, prints a crossed
- *  box of the same dimensions.
- *
- *  The Postscript document is defined by the file pointer <B>fp</B>.
- *  Function based on the DIA source code (http://www.gnome.org/projects/dia)
- *  and licensed under the GNU GPL version 2.
- *
- *  All dimensions are in mils.
- *  
- *  \param [in] toplevel  The TOPLEVEL object.
- *  \param [in] fp         FILE pointer to Postscript document.
- *  \param [in] o_current  Picture OBJECT to write to document.
- *  \param [in] origin_x   Page x coordinate to place picture OBJECT.
- *  \param [in] origin_y   Page y coordinate to place picture OBJECT.
- */
-void o_picture_print(TOPLEVEL *toplevel, FILE *fp, OBJECT *o_current,
-		     int origin_x, int origin_y)
-{
-  int x1, y1, x, y;
-  int height, width;
-  GdkPixbuf* image = o_picture_get_pixbuf (toplevel, o_current);
-  int img_width, img_height, img_rowstride;
-  guint8 *rgb_data;
-  guint8 *mask_data;
-
-  /* calculate the width and height of the box */
-  width  = abs(o_current->picture->lower_x - o_current->picture->upper_x); 
-  height = abs(o_current->picture->upper_y - o_current->picture->lower_y);
-
-  /* calculate the origin of the box */
-  x1 = o_current->picture->upper_x;
-  y1 = o_current->picture->upper_y;
-
-  /* If the image failed to load, try to get hold of the fallback
-   * pixbuf. */
-  if (image == NULL) image = o_picture_get_fallback_pixbuf (toplevel);
-  /* If the image failed to load, draw a box in the default color with a
-   * cross in it. */
-  if (image == NULL) {
-    int line_width = (toplevel->line_style == THICK) ? LINE_WIDTH : 2;
-    o_box_print_solid (toplevel, fp, x1, y1, width, height,
-                       DEFAULT_COLOR, line_width, -1, -1, -1, -1);
-    o_line_print_solid (toplevel, fp, x1, y1, x1+width, y1-height,
-                        DEFAULT_COLOR, line_width, -1, -1, -1, -1);
-    o_line_print_solid (toplevel, fp, x1+width, y1, x1, y1-height,
-                        DEFAULT_COLOR, line_width, -1, -1, -1, -1);
-    return;
-  }
-
-  img_width = gdk_pixbuf_get_width(image);
-  img_rowstride = gdk_pixbuf_get_rowstride(image);
-  img_height = gdk_pixbuf_get_height(image);
-
-  rgb_data = o_picture_rgb_data(image);
-  mask_data = o_picture_mask_data(image);
-
-  fprintf(fp, "gsave\n");
-
-  /* color output only */
-  fprintf(fp, "/pix %i string def\n", img_width * 3);
-  fprintf(fp, "%i %i 8\n", img_width, img_height);
-  fprintf(fp, "%i %i translate\n", x1, y1);
-  fprintf(fp, "%i %i scale\n", width, height);
-  fprintf(fp, "[%i 0 0 -%i 0 0]\n", img_width, img_height);
-
-  fprintf(fp, "{currentfile pix readhexstring pop}\n");
-  fprintf(fp, "false 3 colorimage\n");
-  fprintf(fp, "\n");
-
-  if (mask_data) {
-    for (y = 0; y < img_height; y++) {
-      for (x = 0; x < img_width; x++) {
-	int i = y*img_rowstride+x*3;
-	int m = y*img_width+x;
-        fprintf(fp, "%02x", 255-(mask_data[m]*(255-rgb_data[i])/255));
-        fprintf(fp, "%02x", 255-(mask_data[m]*(255-rgb_data[i+1])/255));
-        fprintf(fp, "%02x", 255-(mask_data[m]*(255-rgb_data[i+2])/255));
-      }
-      fprintf(fp, "\n");
-    }
-  } else {
-    for (y = 0; y < img_height; y++) {
-      for (x = 0; x < img_width; x++) {
-	int i = y*img_rowstride+x*3;
-        fprintf(fp, "%02x", (int)(rgb_data[i]));
-        fprintf(fp, "%02x", (int)(rgb_data[i+1]));
-        fprintf(fp, "%02x", (int)(rgb_data[i+2]));
-      }
-      fprintf(fp, "\n");
-    }
-  }
-  fprintf(fp, "grestore\n");
-  fprintf(fp, "\n");
-   
-  g_object_unref (image);
-  g_free(rgb_data);
-  g_free(mask_data);
-}
-
-
 /*! \brief Embed the image file associated with a picture
  * \par Function Description
  * Verify that a picture has valid data associated with it, and if so,
@@ -948,10 +756,10 @@ void o_picture_print(TOPLEVEL *toplevel, FILE *fp, OBJECT *o_current,
  */
 void o_picture_embed (TOPLEVEL *toplevel, OBJECT *object)
 {
-  const gchar *filename = o_picture_get_filename (toplevel, object);
+  const gchar *filename = o_picture_get_filename (object);
   gchar *basename;
 
-  if (o_picture_is_embedded (toplevel, object)) return;
+  if (o_picture_is_embedded (object)) return;
 
   if (object->picture->file_content == NULL) {
     s_log_message (_("Picture [%s] has no image data.\n"), filename);
@@ -979,10 +787,10 @@ void o_picture_embed (TOPLEVEL *toplevel, OBJECT *object)
 void o_picture_unembed (TOPLEVEL *toplevel, OBJECT *object)
 {
   GError *err = NULL;
-  const gchar *filename = o_picture_get_filename (toplevel, object);
+  const gchar *filename = o_picture_get_filename (object);
   gchar *basename;
 
-  if (!o_picture_is_embedded (toplevel, object)) return;
+  if (!o_picture_is_embedded (object)) return;
 
   o_picture_set_from_file (toplevel, object, filename, &err);
 
@@ -1006,6 +814,7 @@ void o_picture_unembed (TOPLEVEL *toplevel, OBJECT *object)
  *
  *  Interrior points within the picture return a distance of zero.
  *
+ *  \param [in] toplevel     The TOPLEVEL object.
  *  \param [in] object       The picture OBJECT.
  *  \param [in] x            The x coordinate of the given point.
  *  \param [in] y            The y coordinate of the given point.
@@ -1013,8 +822,8 @@ void o_picture_unembed (TOPLEVEL *toplevel, OBJECT *object)
  *  \return The shortest distance from the object to the point. With an
  *  invalid parameter, this function returns G_MAXDOUBLE.
  */
-double o_picture_shortest_distance (OBJECT *object, int x, int y,
-                                    int force_solid)
+double o_picture_shortest_distance (TOPLEVEL *toplevel, OBJECT *object,
+                                    int x, int y, int force_solid)
 {
   double dx, dy;
   double x1, y1, x2, y2;
@@ -1032,7 +841,7 @@ double o_picture_shortest_distance (OBJECT *object, int x, int y,
   dx = min (dx, 0);
   dy = min (dy, 0);
 
-  return sqrt ((dx * dx) + (dy * dy));
+  return hypot (dx, dy);
 }
 
 /*! \brief Test whether a picture object is embedded.
@@ -1041,12 +850,11 @@ double o_picture_shortest_distance (OBJECT *object, int x, int y,
  * in a schematic or symbol file; returns FALSE if its data will be
  * obtained from a separate file.
  *
- * \param toplevel  The current #TOPLEVEL.
  * \param object    The picture #OBJECT to inspect.
  * \return TRUE if \a object is embedded.
  */
 gboolean
-o_picture_is_embedded (TOPLEVEL *toplevel, OBJECT *object)
+o_picture_is_embedded (OBJECT *object)
 {
   g_return_val_if_fail (object != NULL, FALSE);
   g_return_val_if_fail (object->picture != NULL, FALSE);
@@ -1197,12 +1005,11 @@ o_picture_set_from_file (TOPLEVEL *toplevel, OBJECT *object,
  * \par Function Description
  * Returns the filename associated with the picture \a object.
  *
- * \param toplevel The current #TOPLEVEL.
  * \param object   The picture #OBJECT to inspect.
  * \return the filename associated with \a object.
  */
 const gchar *
-o_picture_get_filename (TOPLEVEL *toplevel, OBJECT *object)
+o_picture_get_filename (OBJECT *object)
 {
   g_return_val_if_fail (object != NULL, NULL);
   g_return_val_if_fail (object->picture != NULL, NULL);
