@@ -1,7 +1,7 @@
 /* gEDA - GPL Electronic Design Automation
  * libgeda - gEDA's library
  * Copyright (C) 1998-2010 Ales Hvezda
- * Copyright (C) 1998-2010 gEDA Contributors (see ChangeLog for details)
+ * Copyright (C) 1998-2019 gEDA Contributors (see ChangeLog for details)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,10 +24,6 @@
 
 #include "libgeda_priv.h"
 
-#ifdef HAVE_LIBDMALLOC
-#include <dmalloc.h>
-#endif
-
 /*! \file o_pin_basic.c
  *  \brief functions for the pin object
  */
@@ -53,17 +49,27 @@ void world_get_pin_bounds(TOPLEVEL *toplevel, OBJECT *object, int *left, int *to
  *  \par Function Description
  *  This function gets the position of the whichend side of a pin object.
  *
- *  \param [in] toplevel The toplevel environment.
+ *  \param [in] object   The object to get the position.
  *  \param [out] x       pointer to the x-position
  *  \param [out] y       pointer to the y-position
- *  \param [in] object   The object to get the position.
  *  \return TRUE if successfully determined the position, FALSE otherwise
  */
-gboolean o_pin_get_position (TOPLEVEL *toplevel, gint *x, gint *y,
-                             OBJECT *object)
+gboolean o_pin_get_position (OBJECT *object, gint *x, gint *y)
 {
-  *x = object->line->x[object->whichend];
-  *y = object->line->y[object->whichend];
+  g_return_val_if_fail (object != NULL, FALSE);
+  g_return_val_if_fail (object->type == OBJ_PIN, FALSE);
+  g_return_val_if_fail (object->line != NULL, FALSE);
+  g_return_val_if_fail (object->whichend >= 0, FALSE);
+  g_return_val_if_fail (object->whichend < 2, FALSE);
+
+  if (x != NULL) {
+    *x = object->line->x[object->whichend];
+  }
+
+  if (y != NULL) {
+    *y = object->line->y[object->whichend];
+  }
+
   return TRUE;
 }
 
@@ -72,7 +78,6 @@ gboolean o_pin_get_position (TOPLEVEL *toplevel, gint *x, gint *y,
  *  This function creates and returns a new pin object.
  *  
  *  \param [in]     toplevel    The TOPLEVEL object.
- *  \param [in]     type        The OBJECT type (usually OBJ_PIN)
  *  \param [in]     color       The color of the pin
  *  \param [in]     x1          x-coord of the first point
  *  \param [in]     y1          y-coord of the first point
@@ -83,12 +88,12 @@ gboolean o_pin_get_position (TOPLEVEL *toplevel, gint *x, gint *y,
  *  \return A new pin OBJECT
  */
 OBJECT *o_pin_new(TOPLEVEL *toplevel,
-		  char type, int color,
+		  int color,
 		  int x1, int y1, int x2, int y2, int pin_type, int whichend)
 {
   OBJECT *new_node;
 
-  new_node = s_basic_new_object(type, "pin");
+  new_node = s_basic_new_object(OBJ_PIN, "pin");
   new_node->color = color;
 
   new_node->line = (LINE *) g_malloc(sizeof(LINE));
@@ -100,38 +105,13 @@ OBJECT *o_pin_new(TOPLEVEL *toplevel,
 
   o_pin_set_type (toplevel, new_node, pin_type);
 
-  o_pin_recalc (toplevel, new_node);
+  new_node->w_bounds_valid_for = NULL;
 
   new_node->whichend = whichend;
 
   return new_node;
 }
 
-/*! \brief recalc the visual properties of a pin object
- *  \par Function Description
- *  This function updates the visual coords of the \a o_current object.
- *  
- *  \param [in]     toplevel    The TOPLEVEL object.
- *  \param [in]     o_current   a pin object.
- *
- */
-void o_pin_recalc(TOPLEVEL *toplevel, OBJECT *o_current)
-{
-  int left, right, top, bottom;
-
-  if (o_current->line == NULL) {
-    return;
-  }
-
-  world_get_pin_bounds(toplevel, o_current, &left, &top, &right, &bottom);
-
-  o_current->w_left = left;
-  o_current->w_top = top;
-  o_current->w_right = right;
-  o_current->w_bottom = bottom;
-  o_current->w_bounds_valid = TRUE;
-}
-
 /*! \brief read a pin object from a char buffer
  *  \par Function Description
  *  This function reads a pin object from the buffer \a buf.
@@ -171,7 +151,7 @@ OBJECT *o_pin_read (TOPLEVEL *toplevel, const char buf[],
   }
 
   if (whichend == -1) {
-    s_log_message (_("Found a pin which did not have the whichone field set.\n"
+    s_log_message (_("Found a pin which did not have the whichend field set.\n"
                      "Verify and correct manually.\n"));
   } else if (whichend < -1 || whichend > 1) {
     s_log_message (_("Found an invalid whichend on a pin (reseting to zero): %d\n"),
@@ -179,61 +159,62 @@ OBJECT *o_pin_read (TOPLEVEL *toplevel, const char buf[],
     whichend = 0;
   }
 
-  if (color < 0 || color > MAX_COLORS) {
+  if (color < 0 || color >= MAX_OBJECT_COLORS) {
     s_log_message (_("Found an invalid color [ %s ]\n"), buf);
     s_log_message (_("Setting color to default color\n"));
     color = DEFAULT_COLOR;
   }
 
-  if (toplevel->override_pin_color != -1) {
-    color = toplevel->override_pin_color;
-  }
-
-  new_obj = o_pin_new (toplevel, type, color, x1, y1, x2, y2,
+  new_obj = o_pin_new (toplevel, color, x1, y1, x2, y2,
                        pin_type, whichend);
 
   return new_obj;
 }
 
 /*! \brief Create a string representation of the pin object
- *  \par Function Description
- *  This function takes a pin \a object and return a string
+ *
+ *  This function takes a pin \a object and returns a string
  *  according to the file format definition.
  *
- *  \param [in] toplevel  a TOPLEVEL structure
- *  \param [in] object  a pin OBJECT
- *  \return the string representation of the pin OBJECT
+ *  On failure, this function returns NULL.
+ *
+ *  The caller must free the returned string when no longer needed using
+ *  g_free().
+ *
+ *  \param [in] object a pin object
+ *  \return a string representation of the pin object
  */
-char *o_pin_save(TOPLEVEL *toplevel, OBJECT *object)
+char *o_pin_save(OBJECT *object)
 {
-  int x1, x2, y1, y2;
-  int pin_type, whichend;
-  char *buf;
-  
-  x1 = object->line->x[0];
-  y1 = object->line->y[0];
-  x2 = object->line->x[1];
-  y2 = object->line->y[1];
+  g_return_val_if_fail (object != NULL, NULL);
+  g_return_val_if_fail (object->line != NULL, NULL);
+  g_return_val_if_fail (object->type == OBJ_PIN, NULL);
 
-  pin_type = object->pin_type;
-  whichend = object->whichend;
-  
-  buf = g_strdup_printf("%c %d %d %d %d %d %d %d", object->type,
-		   x1, y1, x2, y2, object->color, pin_type, whichend);
-  return(buf);
+  return g_strdup_printf ("%c %d %d %d %d %d %d %d",
+                          OBJ_PIN,
+                          object->line->x[0],
+                          object->line->y[0],
+                          object->line->x[1],
+                          object->line->y[1],
+                          object->color,
+                          object->pin_type,
+                          object->whichend);
 }
 
 /*! \brief move a pin object
  *  \par Function Description
  *  This function changes the position of a pin \a object.
  *
- *  \param [in] toplevel     The TOPLEVEL object
+ *  \param [ref] object      The pin OBJECT to be moved
  *  \param [in] dx           The x-distance to move the object
  *  \param [in] dy           The y-distance to move the object
- *  \param [in] object       The pin OBJECT to be moved
  */
-void o_pin_translate_world(TOPLEVEL *toplevel, int dx, int dy, OBJECT *object)
+void o_pin_translate_world(OBJECT *object, int dx, int dy)
 {
+  g_return_if_fail (object != NULL);
+  g_return_if_fail (object->line != NULL);
+  g_return_if_fail (object->type == OBJ_PIN);
+
   /* Update world coords */
   object->line->x[0] = object->line->x[0] + dx;
   object->line->y[0] = object->line->y[0] + dy;
@@ -241,9 +222,7 @@ void o_pin_translate_world(TOPLEVEL *toplevel, int dx, int dy, OBJECT *object)
   object->line->y[1] = object->line->y[1] + dy;
 
   /* Update bounding box */
-  o_pin_recalc (toplevel, object);
-
-  s_tile_update_object(toplevel, object);
+  object->w_bounds_valid_for = NULL;
 }
 
 /*! \brief create a copy of a pin object
@@ -258,7 +237,11 @@ OBJECT *o_pin_copy(TOPLEVEL *toplevel, OBJECT *o_current)
 {
   OBJECT *new_obj;
 
-  new_obj = o_pin_new (toplevel, OBJ_PIN, o_current->color,
+  g_return_val_if_fail (o_current != NULL, NULL);
+  g_return_val_if_fail (o_current->line != NULL, NULL);
+  g_return_val_if_fail (o_current->type == OBJ_PIN, NULL);
+
+  new_obj = o_pin_new (toplevel, o_current->color,
                        o_current->line->x[0], o_current->line->y[0],
                        o_current->line->x[1], o_current->line->y[1],
                        o_current->pin_type, o_current->whichend);
@@ -266,44 +249,6 @@ OBJECT *o_pin_copy(TOPLEVEL *toplevel, OBJECT *o_current)
   return new_obj;
 }
 
-/*! \brief postscript print command for a pin object
- *  \par Function Description
- *  This function writes the postscript command of the pin object \a o_current
- *  into the FILE \a fp points to.
- *  
- *  \param [in] toplevel     The TOPLEVEL object
- *  \param [in] fp           pointer to a FILE structure
- *  \param [in] o_current    The OBJECT to print
- *  \param [in] origin_x     x-coord of the postscript origin
- *  \param [in] origin_y     y-coord of the postscript origin
- */
-void o_pin_print(TOPLEVEL *toplevel, FILE *fp, OBJECT *o_current,
-		 int origin_x, int origin_y)
-{
-  int pin_width;
-  int x1, y1;
-  int x2, y2;
-  
-  if (o_current == NULL) {
-    printf("got null in o_pin_print\n");
-    return;
-  }
-
-  f_print_set_color(toplevel, fp, o_current->color);
-
-  x1 = o_current->line->x[0] - origin_x;
-  y1 = o_current->line->y[0] - origin_y;
-  x2 = o_current->line->x[1] - origin_x;
-  y2 = o_current->line->y[1] - origin_y;
-  pin_width = 2;
-  if(toplevel->pin_style == THICK) {
-    pin_width = o_current->line_width;
-  }
-
-  fprintf(fp, "%d %d %d %d %d line\n",x1,y1,x2,y2,pin_width);
-  
-}
-
 /*! \brief rotate a pin object around a centerpoint
  *  \par Function Description
  *  This function rotates a pin \a object around the point
@@ -321,12 +266,16 @@ void o_pin_rotate_world(TOPLEVEL *toplevel, int world_centerx,
 			OBJECT *object)
 {
   int newx, newy;
-	
+
+  g_return_if_fail (object != NULL);
+  g_return_if_fail (object->line != NULL);
+  g_return_if_fail (object->type == OBJ_PIN);
+
   if (angle == 0)
     return;
 
   /* translate object to origin */
-  o_pin_translate_world(toplevel, -world_centerx, -world_centery, object);
+  o_pin_translate_world(object, -world_centerx, -world_centery);
 
   rotate_point_90(object->line->x[0], object->line->y[0], angle,
                   &newx, &newy);
@@ -340,7 +289,7 @@ void o_pin_rotate_world(TOPLEVEL *toplevel, int world_centerx,
   object->line->x[1] = newx;
   object->line->y[1] = newy;
 
-  o_pin_translate_world(toplevel, world_centerx, world_centery, object);
+  o_pin_translate_world(object, world_centerx, world_centery);
 }
 
 /*! \brief mirror a pin object horizontaly at a centerpoint
@@ -356,14 +305,18 @@ void o_pin_rotate_world(TOPLEVEL *toplevel, int world_centerx,
 void o_pin_mirror_world(TOPLEVEL *toplevel,
 			int world_centerx, int world_centery, OBJECT *object)
 {
+  g_return_if_fail (object != NULL);
+  g_return_if_fail (object->line != NULL);
+  g_return_if_fail (object->type == OBJ_PIN);
+
   /* translate object to origin */
-  o_pin_translate_world(toplevel, -world_centerx, -world_centery, object);
+  o_pin_translate_world(object, -world_centerx, -world_centery);
 
   object->line->x[0] = -object->line->x[0];
 
   object->line->x[1] = -object->line->x[1];
 
-  o_pin_translate_world(toplevel, world_centerx, world_centery, object);
+  o_pin_translate_world(object, world_centerx, world_centery);
 }
 
 /*! \brief modify one point of a pin object
@@ -382,12 +335,16 @@ void o_pin_mirror_world(TOPLEVEL *toplevel,
 void o_pin_modify(TOPLEVEL *toplevel, OBJECT *object,
 		  int x, int y, int whichone)
 {
+  g_return_if_fail (object != NULL);
+  g_return_if_fail (object->line != NULL);
+  g_return_if_fail (object->type == OBJ_PIN);
+  g_return_if_fail (object->whichend >= 0);
+  g_return_if_fail (object->whichend < 2);
+
   object->line->x[whichone] = x;
   object->line->y[whichone] = y;
 
-  o_pin_recalc (toplevel, object);
-
-  s_tile_update_object(toplevel, object);
+  object->w_bounds_valid_for = NULL;
 }
 
 /*! \brief guess the whichend of pins of object list
@@ -429,10 +386,8 @@ void o_pin_update_whichend(TOPLEVEL *toplevel,
       while (iter != NULL) {
         o_current = (OBJECT *)iter->data;
         if (o_current->type == OBJ_PIN) {
-          rleft = o_current->w_left;
-          rtop = o_current->w_top;
-          rright = o_current->w_right;
-          rbottom = o_current->w_bottom;
+          (void) world_get_single_object_bounds(
+            toplevel, o_current, &rleft, &rtop, &rright, &rbottom);
 
           if ( found ) {
             left = min( left, rleft );
@@ -463,83 +418,61 @@ void o_pin_update_whichend(TOPLEVEL *toplevel,
       if (o_current->line->y[0] == o_current->line->y[1]) {
         /* horizontal */
         
-        if (o_current->line->x[0] == left) {
-          o_current->whichend = 0;
-        } else if (o_current->line->x[1] == left) {
-          o_current->whichend = 1;        
-        } else if (o_current->line->x[0] == right) {
-          o_current->whichend = 0;        
-        } else if (o_current->line->x[1] == right) {
-          o_current->whichend = 1;
+        d1 = abs(o_current->line->x[0] - left);
+        d2 = abs(o_current->line->x[1] - left);
+        d3 = abs(o_current->line->x[0] - right);
+        d4 = abs(o_current->line->x[1] - right);
+
+        if (d1 <= d2) {
+          min0 = d1;
+          min0_whichend = 0;
         } else {
-            
-          d1 = abs(o_current->line->x[0] - left);
-          d2 = abs(o_current->line->x[1] - left);
-          d3 = abs(o_current->line->x[0] - right);
-          d4 = abs(o_current->line->x[1] - right);
+          min0 = d2;
+          min0_whichend = 1;
+        }
 
-          if (d1 <= d2) {
-            min0 = d1;
-            min0_whichend = 0;
-          } else {
-            min0 = d2;
-            min0_whichend = 1;
-          }
+        if (d3 <= d4) {
+          min1 = d3;
+          min1_whichend = 0;
+        } else {
+          min1 = d4;
+          min1_whichend = 1;
+        }
 
-          if (d3 <= d4) {
-            min1 = d3;
-            min1_whichend = 0;
-          } else {
-            min1 = d4;
-            min1_whichend = 1;
-          }
-
-          if (min0 <= min1) {
-            o_current->whichend = min0_whichend;
-          } else {
-            o_current->whichend = min1_whichend;
-          }
+        if (min0 <= min1) {
+          o_current->whichend = min0_whichend;
+        } else {
+          o_current->whichend = min1_whichend;
         }
            
       } else if (o_current->line->x[0] == o_current->line->x[1]) {
         /* vertical */
         
-        if (o_current->line->y[0] == top) {
-          o_current->whichend = 0;
-        } else if (o_current->line->y[1] == top) {
-          o_current->whichend = 1;        
-        } else if (o_current->line->x[0] == bottom) {
-          o_current->whichend = 0;        
-        } else if (o_current->line->x[1] == bottom) {
-          o_current->whichend = 1;
+        d1 = abs(o_current->line->y[0] - top);
+        d2 = abs(o_current->line->y[1] - top);
+        d3 = abs(o_current->line->y[0] - bottom);
+        d4 = abs(o_current->line->y[1] - bottom);
+
+        if (d1 <= d2) {
+          min0 = d1;
+          min0_whichend = 0;
         } else {
-            
-          d1 = abs(o_current->line->y[0] - top);
-          d2 = abs(o_current->line->y[1] - top);
-          d3 = abs(o_current->line->y[0] - bottom);
-          d4 = abs(o_current->line->y[1] - bottom);
+          min0 = d2;
+          min0_whichend = 1;
+        }
 
-          if (d1 <= d2) {
-            min0 = d1;
-            min0_whichend = 0;
-          } else {
-            min0 = d2;
-            min0_whichend = 1;
-          }
+        if (d3 <= d4) {
+          min1 = d3;
+          min1_whichend = 0;
+        } else {
+          min1 = d4;
+          min1_whichend = 1;
+        }
 
-          if (d3 <= d4) {
-            min1 = d3;
-            min1_whichend = 0;
-          } else {
-            min1 = d4;
-            min1_whichend = 1;
-          }
-
-          if (min0 <= min1) {
-            o_current->whichend = min0_whichend;
-          } else {
-            o_current->whichend = min1_whichend;
-          }
+        if (min0 <= min1) {
+          o_current->whichend = min0_whichend;
+        } else {
+          o_current->whichend = min1_whichend;
         }
       }
     }
@@ -559,6 +492,9 @@ void o_pin_update_whichend(TOPLEVEL *toplevel,
  */
 void o_pin_set_type (TOPLEVEL *toplevel, OBJECT *o_current, int pin_type)
 {
+  g_return_if_fail (o_current != NULL);
+  g_return_if_fail (o_current->type == OBJ_PIN);
+
   o_emit_pre_change_notify (toplevel, o_current);
   switch (pin_type) {
     default:
