@@ -1,6 +1,6 @@
 /* gEDA - GPL Electronic Design Automation
  * libgeda - gEDA's library
- * Copyright (C) 1998-2010 gEDA Contributors (see ChangeLog for details)
+ * Copyright (C) 1998-2019 gEDA Contributors (see ChangeLog for details)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -111,7 +111,6 @@
  */
 
 #include <config.h>
-#include <missing.h>
 
 #include <stdio.h>
 #include <glib.h>
@@ -120,10 +119,6 @@
 #include <string.h>
 #endif
 
-#ifdef HAVE_LIBDMALLOC
-#include <dmalloc.h>
-#endif
-
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #else
@@ -250,6 +245,9 @@ static gchar *get_data_directory (const CLibSymbol *symbol);
 static gchar *get_data_command (const CLibSymbol *symbol);
 static gchar *get_data_scm (const CLibSymbol *symbol);
 
+static gboolean s_clib_updating = FALSE;
+static GList *s_clib_update_closures = NULL;
+
 /*! \brief Initialise the component library.
  *  \par Function Description
  *  Resets and initialises the component library.
@@ -362,6 +360,8 @@ void s_clib_free ()
     g_list_free (clib_sources);
     clib_sources = NULL;
   }
+
+  s_clib_begin_update ();
 }
 
 /*! \brief Compare two component sources by name.
@@ -586,6 +586,8 @@ static void refresh_directory (CLibSource *source)
   g_return_if_fail (source != NULL);
   g_return_if_fail (source->type == CLIB_DIR);
 
+  s_clib_begin_update ();
+
   /* Clear the current symbol list */
   g_list_foreach (source->symbols, (GFunc) free_symbol, NULL);
   g_list_free (source->symbols);
@@ -661,6 +663,8 @@ static void refresh_command (CLibSource *source)
   g_return_if_fail (source != NULL);
   g_return_if_fail (source->type == CLIB_CMD);
 
+  s_clib_begin_update ();
+
   /* Clear the current symbol list */
   g_list_foreach (source->symbols, (GFunc) free_symbol, NULL);
   g_list_free (source->symbols);
@@ -723,6 +727,8 @@ static void refresh_scm (CLibSource *source)
   g_return_if_fail (source != NULL);
   g_return_if_fail (source->type == CLIB_SCM);
 
+  s_clib_begin_update ();
+
   /* Clear the current symbol list */
   g_list_foreach (source->symbols, (GFunc) free_symbol, NULL);
   g_list_free (source->symbols);
@@ -730,13 +736,13 @@ static void refresh_scm (CLibSource *source)
 
   symlist = scm_call_0 (source->list_fn);
 
-  if (SCM_NCONSP (symlist) && (symlist != SCM_EOL)) {
+  if (!scm_is_pair (symlist) && !scm_is_null (symlist)) {
     s_log_message (_("Failed to scan library [%s]: Scheme function returned non-list\n"),
 		   source->name);
     return;
   }
 
-  while (symlist != SCM_EOL) {
+  while (!scm_is_null (symlist)) {
     symname = SCM_CAR (symlist);
     if (!scm_is_string (symname)) {
       s_log_message (_("Non-string symbol name while scanning library [%s]\n"),
@@ -772,9 +778,6 @@ static void refresh_scm (CLibSource *source)
  *  Resets the list of symbols available from each source, and
  *  repopulates it from scratch.  Useful e.g. for checking for new
  *  symbols.
- *
- *  \bug Disabled for now because it would break cached CLibSymbols used
- *  all over the place (e.g. in #st_object).
  */
 void s_clib_refresh ()
 {
@@ -803,6 +806,8 @@ void s_clib_refresh ()
         break;
       }
   }
+
+  s_clib_end_update ();
 }
 
 /*! \brief Get a named component source.
@@ -1042,6 +1047,14 @@ gchar *s_clib_symbol_get_filename (const CLibSymbol *symbol)
   return g_build_filename(symbol->source->directory, symbol->name, NULL);
 }
 
+const gchar *s_clib_source_get_directory (const CLibSource *source)
+{
+  if (source->type != CLIB_DIR)
+    return NULL;
+
+  return source->directory;
+}
+
 /*! \brief Get the source to which a symbol belongs.
  *  \par Function Description
  *  Get the source which a symbol is associated.
@@ -1496,3 +1509,64 @@ GList *s_toplevel_get_symbols (const TOPLEVEL *toplevel)
 
   return result;
 }
+
+
+
+/*! \brief A marshaller for a GCClosure with a callback of type
+ *         `void (*callback) (gpointer user_data)`.
+ */
+static void
+update_marshal (GClosure *closure,
+                GValue *return_value,
+                guint n_param_values,
+                const GValue *param_values,
+                gpointer invocation_hint,
+                gpointer marshal_data)
+{
+  GCClosure *cclosure = (GCClosure *) closure;
+  g_return_if_fail (n_param_values == 0);
+
+  typedef void (*marshal_func) (gpointer data);
+  marshal_func callback = (marshal_func) (marshal_data ? marshal_data
+                                                       : cclosure->callback);
+  callback (closure->data);
+}
+
+void s_clib_add_update_callback (void (*update) (gpointer user_data),
+                                 gpointer user_data)
+{
+  GClosure *closure = g_cclosure_new (G_CALLBACK (update), user_data, NULL);
+  g_closure_set_marshal (closure, update_marshal);
+
+  s_clib_update_closures = g_list_append (s_clib_update_closures, closure);
+  g_closure_ref (closure);
+  g_closure_sink (closure);
+}
+
+void s_clib_remove_update_callback (gpointer user_data)
+{
+  for (GList *l = s_clib_update_closures; l != NULL; l = l->next) {
+    GClosure *closure = (GClosure *) l->data;
+    if (closure->data != user_data)
+      continue;
+
+    s_clib_update_closures = g_list_remove (s_clib_update_closures, closure);
+    g_closure_unref (closure);
+    break;
+  }
+}
+
+void s_clib_begin_update ()
+{
+  s_clib_updating = TRUE;
+}
+
+void s_clib_end_update ()
+{
+  if (!s_clib_updating)
+    return;
+  s_clib_updating = FALSE;
+
+  for (GList *l = s_clib_update_closures; l != NULL; l = l->next)
+    g_closure_invoke ((GClosure *) l->data, NULL, 0, NULL, NULL);
+}
