@@ -1,7 +1,7 @@
 /* gEDA - GPL Electronic Design Automation
  * gschlas - gEDA Load and Save
  * Copyright (C) 2002-2010 Ales Hvezda
- * Copyright (C) 2002-2010 gEDA Contributors (see ChangeLog for details)
+ * Copyright (C) 2002-2019 gEDA Contributors (see ChangeLog for details)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
  */
 
 #include <config.h>
-#include <missing.h>
 #include <version.h>
 
 #include <stdio.h>
@@ -39,50 +38,47 @@
 #include <libgeda/libgeda.h>
 
 #include "../include/globals.h"
-#include "../include/i_vars.h"
 #include "../include/prototype.h"
 
-#ifdef HAVE_LIBDMALLOC
-#include <dmalloc.h>
-#endif
-
+/*! \brief Test the version of gschlas and gEDA/gaf
+ *
+ * \param version Version being tested
+ * \returns false if incorrect version, true if OK
+ */
 SCM g_rc_gschlas_version(SCM scm_version)
 {
     char *version;
     SCM ret = SCM_BOOL_T;
+    SCM rc_filename;
+    char *sourcefile;
 
     SCM_ASSERT (scm_is_string (scm_version), scm_version,
-		SCM_ARG1, "gschlas-version");
+                SCM_ARG1, "gschlas-version");
 
+    scm_dynwind_begin (0);
     version = scm_to_utf8_string (scm_version);
-    if (g_strcasecmp (version, PACKAGE_DATE_VERSION) != 0) {
+    scm_dynwind_free (version);
+
+    if (g_ascii_strcasecmp (version, PACKAGE_DATE_VERSION) != 0) {
+      sourcefile = NULL;
+      rc_filename = g_rc_rc_filename ();
+      if (scm_is_false (rc_filename)) {
+        rc_filename = scm_from_utf8_string ("unknown");
+      }
+      sourcefile = scm_to_utf8_string (rc_filename);
+      scm_dynwind_free (sourcefile);
       fprintf(stderr,
               "You are running gEDA/gaf version [%s%s.%s],\n",
               PREPEND_VERSION_STRING, PACKAGE_DOTTED_VERSION,
               PACKAGE_DATE_VERSION);
       fprintf(stderr,
               "but you have a version [%s] gschlasrc file:\n[%s]\n",
-              version, rc_filename);
+              version, sourcefile);
       fprintf(stderr,
               "Please be sure that you have the latest rc file.\n");
       ret = SCM_BOOL_F;
     }
+    scm_dynwind_end();
 
-    free (version);
     return ret;
 }
-
-
-SCM 
-g_rc_force_boundingbox(SCM mode)
-{
-  static const vstbl_entry mode_table[] = {
-    {TRUE, "enabled"},
-    {FALSE, "disabled"}
-			    };
-
-  RETURN_G_RC_MODE("force-boundingbox", default_force_boundingbox, 2);
-}
-
-
-/*************************** GUILE end done *********************************/
